package edu.umich.popcup.android;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.util.AttributeSet;
import android.util.Log;
import android.view.ContextMenu;
import android.view.Display;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.view.ContextMenu.ContextMenuInfo;
import android.widget.LinearLayout;
import edu.umich.popcup.util.Image;

public class ImageDashboard extends Activity {
	
	private DrawableImageView mImageView;
	private Uri mImageUri;
	private Bitmap mImageBitmap;
	private Boolean mImageReady = false;
	
	private static final int DEFAULT_CAMERA = 0;
	private static final String TAG = "PopCup";
	
	public static final int TAKE_PICTURE = Menu.FIRST;
	public static final int SEND_PICTURE = Menu.FIRST + 1;
	
	public static final int SELECT_POSE = Menu.FIRST + 2;
	public static final int SELECT_POSE_FRONT = Menu.FIRST + 3;
	public static final int SELECT_POSE_BACK = Menu.FIRST + 4;
	public static final int SELECT_POSE_UP = Menu.FIRST + 5;
	public static final int SELECT_POSE_BELOW = Menu.FIRST + 6;
	public static final int SELECT_POSE_RIGHT = Menu.FIRST + 7;
	public static final int SELECT_POSE_LEFT = Menu.FIRST + 8;
	
	public static final int SELECT_BB = Menu.FIRST + 9;
	
	public static final String IMAGE_URI = "image_uri";
	public static final String IMAGE_READY = "image_ready";
	public static final String IMAGE_INFO = "image_info";
	
	private Image mImage;
	private boolean mBBoxMode; // To record whether it is in bounding box mode
	private float [][] mBBoxPoints;
	private int mImageRotation;
	


    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	
	        super.onCreate(savedInstanceState);
	        new LinearLayout(this);
	        //mImageView = new ImageView(this);
	        mImage = new Image();
	        
	        try {
				mImageUri = Uri.fromFile(new File(Environment.getExternalStorageDirectory().getCanonicalPath() + "/test.jpg"));
			} catch (IOException e) {
				e.printStackTrace();
			}
			
	        setContentView(R.layout.image_dashboard);
	        mImageView = (DrawableImageView)findViewById(R.id.drawable_image_view);
	        mImageView.setOnTouchListener(new View.OnTouchListener() {
				
				public boolean onTouch(View v, MotionEvent event) {
					if (mBBoxMode) {
						if (event.getAction() == MotionEvent.ACTION_DOWN) {
							return true;
						}
						if (event.getAction() == MotionEvent.ACTION_UP) {
							if (mBBoxPoints[0] == null) {
								mBBoxPoints[0] = new float[2];
								mBBoxPoints[0][0] = event.getX();
								mBBoxPoints[0][1] = event.getY();
							} else if (mBBoxPoints[1] == null) {
								mBBoxPoints[1] = new float[2];
								mBBoxPoints[1][0] = event.getX();
								mBBoxPoints[1][1] = event.getY();
							} else {
								mBBoxMode = false;
								return false;
							}
							return true;
						}
						mImageView.invalidate();
					}

					return false;
				}
			});
			registerForContextMenu(mImageView);
	        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
	                WindowManager.LayoutParams.FLAG_FULLSCREEN);
        
			mBBoxMode = false;
			mBBoxPoints = new float[2][];
    }
    
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
    	super.onActivityResult(requestCode, resultCode, data);
    	if (resultCode != RESULT_OK) {
    		Log.i(TAG, "Image result is not OK. it is" + resultCode);
    		mImageReady = false;
    		return;
    	} else {
    		//Bitmap image = null;
    		/*try {
    			mImage = MediaStore.Images.Media.getBitmap(getContentResolver(), mImageUri);
    			Log.i(TAG, "Image is opened successfully");
    			mImageView.setImageBitmap(mImage);
    		} catch (FileNotFoundException e) {
    			e.printStackTrace();
    		} catch (IOException e) {
    			e.printStackTrace();
    		}*/
    		Log.i(TAG, "Successfully capture image");
    		mImageReady = true;
    		Display display = ((WindowManager) getSystemService(WINDOW_SERVICE)).getDefaultDisplay();
    		mImageRotation = display.getOrientation();
    		return;
    		//mImageCanvas = new Canvas(image);
    		//mImageView.draw(new Canvas(image));
    	}
    }
    
    @Override
    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
    	super.onCreateContextMenu(menu, v, menuInfo);
    	MenuInflater inflater = getMenuInflater();
    	inflater.inflate(R.menu.image_view_cmenu, menu);
    }
    
	@Override
	public boolean onContextItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		
		// For pose options
		case R.id.select_pose_back:
			mImage.setPose(Image.Pose.BACK);
			return true;
		case R.id.select_pose_below:
			mImage.setPose(Image.Pose.BELOW);
			return true;
		case R.id.select_pose_front:
			mImage.setPose(Image.Pose.FRONT);
			return true;
		case R.id.select_pose_above:
			mImage.setPose(Image.Pose.ABOVE);
			return true;
		case R.id.select_pose_left:
			mImage.setPose(Image.Pose.LEFT);
			return true;
		case R.id.select_pose_right:
			mImage.setPose(Image.Pose.RIGHT);
			return true;
			
		// For Category options
		case R.id.select_category_cup:
			mImage.setCategory(Image.Category.CUP);
			return true;
		case R.id.select_category_mouse:
			mImage.setCategory(Image.Category.MOUSE);
			return true;
		case R.id.select_category_stapler:
			mImage.setCategory(Image.Category.STAPLER);
			return true;
			
		// For Selecting bounding box
		case R.id.select_bbox:
			mBBoxMode = true;
			mBBoxPoints = new float[2][];
			mImageView.setBBoxMode(mBBoxMode);
			mImageView.setBBoxPoints(mBBoxPoints);
			mImageView.invalidate();
			return true;
		default:
			return super.onContextItemSelected(item);
		}
	}
    
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu){
    	boolean result = super.onCreateOptionsMenu(menu);
    	menu.add(0, SEND_PICTURE, 0, R.string.menu_send_picture);
    	menu.add(0, TAKE_PICTURE, 0, R.string.menu_take_picture);
    	return result;
    }
    
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
    	boolean result = super.onOptionsItemSelected(item);
    	Intent intent = null;
    	
    	switch (item.getItemId()) {
    	case SEND_PICTURE:
    		intent = new Intent(this, SendImage.class);
    		if (mBBoxPoints[0] == null) 
    			mBBoxPoints[0] = new float[2];
    		if (mBBoxPoints[1] == null) 
    			mBBoxPoints[1] = new float[2];
    		Image.BoundingBox bBox = new Image.BoundingBox(mBBoxPoints[0][0], mBBoxPoints[0][1], mBBoxPoints[1][0], mBBoxPoints[1][1]);
    		mImage.setBoundingBox(bBox);
    		intent.putExtra(PopCup.IMAGE_URI, mImageUri);
    		intent.putExtra(PopCup.IMAGE_READY, mImageReady);
    		intent.putExtra(ImageDashboard.IMAGE_INFO, mImage);
    		try {
    			startActivity(intent);
    		} catch (Exception e) {
    			e.printStackTrace();
    		}
    		break;
    	case TAKE_PICTURE:
    		//mImage.setImageStream(new FileInputStream(mImageUri));
    		intent = new Intent("android.media.action.IMAGE_CAPTURE");
			intent.putExtra(MediaStore.EXTRA_OUTPUT, mImageUri);
			startActivityForResult(intent, DEFAULT_CAMERA);
    		break;
    	}
    	return result;
    }

	@Override
	protected void onPause() {
		super.onPause();
	}

	@Override
	protected void onResume() {
		super.onResume();
		try {
			mImageBitmap = MediaStore.Images.Media.getBitmap(getContentResolver(), mImageUri);
		} catch (FileNotFoundException e) {
			Log.i(TAG, "The image to show is not found");
			e.printStackTrace();
		} catch (IOException e) {
			Log.i(TAG, "I/O exception when opening the image");
			e.printStackTrace();
		}
		
		//Canvas canvas = new Canvas();
		//Canvas c = canvas;
		//mImageView.setImageBitmap(mImageBitmap);	
		//mImageView.setImageDrawable(Drawable.createFromPath(mImageUri.getPath()));
		mImageView.setImage(mImageBitmap);
	}

	@Override
	protected void onStart() {
		super.onStart();
	}

	@Override
	protected void onStop() {
		super.onStop();
		//mImageBitmap.recycle();
	}
}

class DrawableImageView extends View {
	
	private float [][] mBBoxPoints;
	private boolean mBBoxMode;
	private Bitmap mImage;

	public DrawableImageView(Context context) {
		super(context);
		mImage = null;
	}
	
	public DrawableImageView(Context context, AttributeSet attrs) {
		super(context, attrs);
		mImage = null;
	}

	@Override
	public void draw(Canvas canvas) {
		super.draw(canvas);
		if (mImage != null)
			canvas.drawBitmap(mImage, null, new Rect(0, 0, canvas.getWidth(), canvas.getHeight()), null);
		if (mBBoxMode) {
			Paint paint = new Paint();
			paint.setStrokeWidth(10);
			paint.setColor(0xFFFFFF00);
			paint.setStyle(Paint.Style.STROKE);
			if (mBBoxPoints[0] != null && mBBoxPoints[1] != null) {
				float left = (mBBoxPoints[0][0] < mBBoxPoints[1][0]) ? mBBoxPoints[0][0] : mBBoxPoints[1][0];
				float right = (mBBoxPoints[0][0] > mBBoxPoints[1][0]) ? mBBoxPoints[0][0] : mBBoxPoints[1][0];
				float top = (mBBoxPoints[0][1] < mBBoxPoints[1][1]) ? mBBoxPoints[0][1] : mBBoxPoints[1][1];
				float bottom = (mBBoxPoints[0][1] > mBBoxPoints[1][1]) ? mBBoxPoints[0][1] : mBBoxPoints[1][1];
				canvas.drawRect(left, top, right, bottom, paint);
			} else {
				if (mBBoxPoints[0] != null)
					canvas.drawPoints(mBBoxPoints[0], paint);
				if (mBBoxPoints[1] != null)
					canvas.drawPoints(mBBoxPoints[1], paint);
			}
		}
	}

	public void setImage(Bitmap image) {
		mImage = image;
	}
	public void setBBoxMode(boolean bBoxMode) {
		mBBoxMode = bBoxMode;
	}
	
	public void setBBoxPoints(float [][] bBoxPoints) {
		mBBoxPoints = bBoxPoints;
	}
}
