package edu.umich.popcup.android;

import java.io.ByteArrayInputStream;
import java.nio.Buffer;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.nio.ShortBuffer;

import javax.microedition.khronos.opengles.GL10;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.opengl.GLUtils;

public class Mesh {
	
	private FloatBuffer vertexBuffer;
	private IntBuffer colorBuffer;
	private ShortBuffer indexBuffer;
	private FloatBuffer texCoordBuffer;
	
	private int texId[];
	private int texWidth;
	private int texHeight;
	private Bitmap texture; 
	
	private float xMean;
	private float yMean;
	private float zMean;
	
	private int numOfIndices;
	
	private static Buffer extract(Bitmap bmp) 
	{ 
		ByteBuffer bb = ByteBuffer.allocateDirect(bmp.getHeight() * bmp.getWidth() * 4); 
		bb.order(ByteOrder.nativeOrder()); 
		IntBuffer ib = bb.asIntBuffer(); 
		// Convert ARGB -> RGBA 
		for (int y = bmp.getHeight() - 1; y > -1; y--) 
		{ 

			for (int x = 0; x < bmp.getWidth(); x++) 
			{ 
				int pix = bmp.getPixel(x, bmp.getHeight() - y - 1); 
				int alpha = ((pix >> 24) & 0xFF); 

				// Make up alpha for interesting effect 

				//ib.put(red << 24 | green << 16 | blue << 8 | ((red + blue + green) / 3)); 
				ib.put(pix << 8 | alpha); 
			} 
		} 
		//bb.position(0);
		ib.position(0);
		return ib; 
	}
	
	public Mesh() {
		super();
	}
	
	public void setVertices(float [] vertices) {
		ByteBuffer vbb = ByteBuffer.allocateDirect(vertices.length * 4);
		vbb.order(ByteOrder.nativeOrder());
		vertexBuffer = vbb.asFloatBuffer();
		vertexBuffer.put(vertices);
		vertexBuffer.position(0);
		
		for (int i = 0; i < vertices.length; i+= 3) {
			xMean += vertices[i];
			yMean += vertices[i+1];
			zMean += vertices[i+2];
		}
		
		int len = vertices.length / 3;
		xMean /= len;
		yMean /= len;
		zMean /= len;
	}
	
	public void setColors(int [] colors) {
		ByteBuffer cbb = ByteBuffer.allocateDirect(colors.length * 4);
		cbb.order(ByteOrder.nativeOrder());
		colorBuffer = cbb.asIntBuffer();
		colorBuffer.put(colors);
		colorBuffer.position(0);
	}
	
	public void setIndices(short [] indices) {
		ByteBuffer ibb = ByteBuffer.allocateDirect(indices.length * 2);
		ibb.order(ByteOrder.nativeOrder());
		indexBuffer = ibb.asShortBuffer();
		indexBuffer.put(indices);
		indexBuffer.position(0);
		numOfIndices = indices.length;
	}
	
	public void setTexCoords(float [] texCoords) {
		ByteBuffer tbb = ByteBuffer.allocateDirect(texCoords.length * 4);
		tbb.order(ByteOrder.nativeOrder());
		texCoordBuffer = tbb.asFloatBuffer();
		texCoordBuffer.put(texCoords);
		texCoordBuffer.position(0);
	}
	
	public void setTexture(byte [] texBytes) {
		texId = new int[1];
		try {
			//ByteArrayInputStream bis = new ByteArrayInputStream(texBytes);
			/*String imagePath = Environment.getExternalStorageDirectory().getCanonicalPath() + "/texture.jpg";
			FileOutputStream fos = new FileOutputStream(imagePath);
			fos.write(texBytes);
			fos.close();*/
			texture = BitmapFactory.decodeStream(new ByteArrayInputStream(texBytes));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public float getXMean() {
		return xMean;
	}
	
	public float getYMean() {
		return yMean;
	}
	
	public float getZMean() {
		return zMean;
	}
	
	public void init(GL10 gl) {
		if (texId != null) {
			gl.glGenTextures(1, texId, 0);
			gl.glBindTexture(GL10.GL_TEXTURE_2D, texId[0]);
        	gl.glTexParameterx(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MIN_FILTER, GL10.GL_LINEAR); 
        	gl.glTexParameterx(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MAG_FILTER, GL10.GL_LINEAR);
    		gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_WRAP_S, GL10.GL_REPEAT);
    		gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_WRAP_T, GL10.GL_REPEAT);
    		GLUtils.texImage2D(GL10.GL_TEXTURE_2D, 0, texture, 0);
    		int width = texture.getWidth();
    		//System.out.println(width);
		}
	}
	
	public void draw(GL10 gl) {
		boolean isTexOn = false;
        // Counter-clockwise winding.
		gl.glFrontFace(GL10.GL_CCW);
		// Enable face culling.
		//gl.glEnable(GL10.GL_CULL_FACE);
		// What faces to remove with the face culling.
		//gl.glCullFace(GL10.GL_BACK);
		// Enabled the vertices buffer for writing and to be used during
		// rendering.
		gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
		// Specifies the location and data format of an array of vertex
		// coordinates to use when rendering.
		gl.glVertexPointer(3, GL10.GL_FLOAT, 0, vertexBuffer);
	        // Set flat color
	        //gl.glColor4f(rgba[0], rgba[1], rgba[2], rgba[3]);
	        // Smooth color
        if ( colorBuffer != null ) {
            // Enable the color array buffer to be used during rendering.
            gl.glEnableClientState(GL10.GL_COLOR_ARRAY);
            // Point out the where the color buffer is.
            gl.glColorPointer(4, GL10.GL_FIXED, 0, colorBuffer);
        } else {
        	gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
        }
    	if (texCoordBuffer != null) {
    		gl.glBindTexture(GL10.GL_TEXTURE_2D, texId[0]);
    		gl.glEnableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
    		gl.glTexCoordPointer(2, GL10.GL_FLOAT, 0, texCoordBuffer);
    		isTexOn = true;
    	}
		gl.glDrawElements(GL10.GL_TRIANGLES, numOfIndices,
			GL10.GL_UNSIGNED_SHORT, indexBuffer);
		// Disable the vertices buffer.
		gl.glDisableClientState(GL10.GL_VERTEX_ARRAY);
		if (colorBuffer != null) {
			gl.glDisableClientState(GL10.GL_COLOR_ARRAY);
		}
		if (isTexOn) {
			gl.glDisableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
		}
		// Disable face culling.
		//gl.glDisable(GL10.GL_CULL_FACE);
	}
}
